/* Text clustering method, SKWIC algorithm implementation
 * Controller for application graphics
 * Authors: Roman Milos, Vladimir Bartik, May 2011
 */
package dip.kontroler;

import dip.Main;
import dip.pohled.InitOkno;
import dip.pohled.AlgoritmusOkno;
import dip.pohled.VysledkyPredzpracovaniOkno;
import dip.model.Data;
import dip.model.IO;
import dip.model.Predzpracovani;
import dip.model.algoritmus.Algoritmus;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.JFrame;
import javax.swing.ProgressMonitor;
import javax.swing.SwingWorker;
import javax.swing.Timer;
import javax.swing.UIManager;
import javax.xml.bind.JAXBException;

/**
 * Controller class to process graphics of the application. 
 */
public class Gui {

    private Timer timer;
    //interval between timer events
    public final static int CEKEJ = 50;
    //monitoring of complex poperations
    private ProgressMonitor progressMonitor;
    //complex task
    private Task task;
    //detection of state
    boolean predzpracovani = false;
    boolean krok = false;
    boolean inicializace = false;
    boolean beh = false;
    //Singleton GUI instance
    private static Gui gui = new Gui();

    /**
     * Get instance of this class as a singleton.
     */
    public static Gui getInstance() {

        return gui;
    }
    // data storage
    private Data data;
    //pre-processing class
    private Predzpracovani predzprac;
    //mining algorithm class
    private Algoritmus alg;

/**
 * Get instance of algorithm class.
 * @return instance of algorithm class
 */
    public Algoritmus getAlg() {
        return alg;
    }
    
/**
 * Sets the algorithm.
 * @param instance of algorithm class
 */
    public void setAlg(Algoritmus alg) {
        this.alg = alg;
    }
    
/**
 * Gets the input dataset.
 * @return input dataset
 */
    public Data getData() {
        return data;
    }
    
/**
 * Sets the input dataset.
 * @param instance of class representing input data
 */
    public void setData(Data data) {
        this.data = data;
    }
    
    //Application windows
    public InitOkno initOkno;
    public VysledkyPredzpracovaniOkno predzpracovaniOkno;
    public AlgoritmusOkno algoritmusOkno;

    /**
     * Constructor. Creates all necessary windows, hides them and initializes 
     * variables. According to data, displays suitable window.
     */
    public Gui() {

        predzprac = Predzpracovani.getInstance();
        data = predzprac.getData();
        this.lokalizujFileChooser();

        initOkno = new InitOkno(this);
        predzpracovaniOkno = new VysledkyPredzpracovaniOkno(this);
        algoritmusOkno = new AlgoritmusOkno(this);


        timer = new Timer(CEKEJ, new TimerListener());
        if (data.getDokumenty().size() > 1 && data.getSlovaDokumenty().size() > 1) {
            predzprac.opravID
						();
            predzprac.setZmena(false);
            dokonciPredzpracovani();
        } else {
            predzprac.setZmena(true);
            spustitInicializaci();

        }
    }
    
    /**
     * Executes initialization window. Data are actualized and other windows 
     * are hidden.
     */
    public void spustitInicializaci() {
        boolean tmp = predzprac.isZmena();
        initOkno.nastav();
        predzprac.setZmena(tmp);
        predzpracovaniOkno.setVisible(false);
        initOkno.setVisible(true);

        algoritmusOkno.setVisible(false);
        predzpracovaniOkno.setVybranyDokument(null);

    }
    
   /**
     * Executes algorithm results window. Data are actualized and other windows 
     * are hidden.
     */
    public void spustitVysledky() {
        algoritmusOkno.nastav();
        algoritmusOkno.setVisible(true);
        initOkno.setVisible(false);
        predzpracovaniOkno.setVisible(false);


    }

   /**
     * Executes pre-processing results window. Data are actualized and other 
     * windows are hidden.
     */
    public void dokonciPredzpracovani() {


        predzpracovaniOkno.nastav();
        predzpracovaniOkno.setVisible(true);
        initOkno.setVisible(false);
        algoritmusOkno.setVisible(false);

    }

   /**
     * Localization of the file chooser.
     */
    private void lokalizujFileChooser() {
        UIManager.put("FileChooser.cancelButtonText", "Cancel");
        UIManager.put("FileChooser.cancelButtonToolTipText", "Cancel");
        UIManager.put("FileChooser.fileNameLabelText", "File name:");
        UIManager.put("FileChooser.acceptAllFileFilterText", "more");
        UIManager.put("FileChooser.lookInLabelText", "Searching area");
        // UIManager.put("FileChooser.openButtonText", "Vyber");
        UIManager.put("FileChooser.openButtonToolTipText", "Choose Input data");
        UIManager.put("FileChooser.filesOfTypeLabelText", "File types");
        UIManager.put("FileChooser.listViewButtonToolTipText", "Icons");
        UIManager.put("FileChooser.listViewButtonAccessibleName", "Icons displaying");
        UIManager.put("FileChooser.detailsViewButtonToolTipText", "List");
        UIManager.put("FileChooser.detailsViewButtonAccessibleName", "List displaying");
        UIManager.put("FileChooser.upFolderToolTipText", "One level up");
        UIManager.put("FileChooser.upFolderAccessibleName", "One level up");
        UIManager.put("FileChooser.homeFolderToolTipText", "Home folder");
        UIManager.put("FileChooser.homeFolderAccessibleName", "Home folder");
        UIManager.put("FileChooser.openButtonText", "Open");
        UIManager.put("FileChooser.openDialogTitleText", "Open");
        UIManager.put("ProgressMonitor.progressText", "Waiting for execution");
        UIManager.put("OptionPane.cancelButtonText", "Cancel");

    }
    
   /**
     * Makes export into the XML file. 
     * @param frame a JFrame instance
     * @param soubor output XML file
     */
    public void xmlExport(JFrame frame, File soubor) {
        try {
            frame.setCursor(new Cursor(Cursor.WAIT_CURSOR));
            frame.setCursor(new Cursor(Cursor.WAIT_CURSOR));

            if (predzprac.isZmena()) {
                data.reInit();
            }
            if (soubor != null) {
                IO.export(soubor);
            } else {
                throw new Exception("No file to write is chosen");
            }

        } catch (Exception e) {
            Main.vyridVyjimku(e);
        } finally {
            frame.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        }


    }
    
   /**
     * Imports an XML file and sends it to the view layer classes. 
     * @param frame a JFrame instance
     * @param soubor input XML file
     */
    public void xmlImport(JFrame frame, File soubor) {


        try {
            frame.setCursor(new Cursor(Cursor.WAIT_CURSOR));

            IO.importuj(soubor);
        } catch (IOException ex) {
            Main.vyridVyjimku(new Exception("Unable to get import file"));
            return;
        } catch (JAXBException ex) {
            Main.vyridVyjimku(new Exception("Incorrect format of XML file"));
            return;
        } catch (Exception ex) {
            ex.printStackTrace();
            Main.vyridVyjimku(new Exception(ex));

            return;
        } finally {
            frame.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        }


        if (data.getDokumenty().size() > 1 && data.getSlovaDokumenty().size() > 1) {
            predzprac.opravID();

            dokonciPredzpracovani();
        } else {
            predzprac.setZmena(true);
            spustitInicializaci();
        }

    }

    /**
     * Makes centering of selected window.
     * @param okno a Frame instance
     */
    public void centruj(JFrame okno) {
        Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
        okno.setLocation(((int) dim.getWidth() - okno.getWidth()) >> 1, ((int) dim.getHeight() - okno.getHeight()) >> 1);
    }

    /**
     * A class for a complicated task, which is executed on the background.  
     * Runs the task and waits for eventual exceptions.
     */
    class Task extends SwingWorker<Void, Void> {

        @Override
        public Void doInBackground() throws Exception {
            // setProgress(0);
            try {
                if (krok) {
                    alg.krok();
                } else if (predzpracovani) {
                    predzprac.beh();
                } else if (beh) {
                    alg.beh();
                } else if (inicializace) {
                    alg.inicializace();
                }
            } catch (Exception e) {

                if (predzpracovani) {
                    predzprac.storno = true;
                    initOkno.setVisible(true);
                } else {
                    alg.setStorno(true);
                    //  algoritmusOkno.setVisible(true);
                }
                Main.vyridVyjimku(e);
                this.cancel(true);
            }
            return null;
        }

        @Override
        public void done() {

            if (!predzpracovani) {
                algoritmusOkno.nastav();
            }
        }
    }

    /**
     * A class realizing the timer. By each event checks, if something 
     * was canceled and stops the task. Otherwise it finishes the task. 
     */
    class TimerListener implements ActionListener {

        public void actionPerformed(ActionEvent evt) {

            if (progressMonitor.isCanceled()) {

                progressMonitor.close();
                if (inicializace) {
                    dokonciPredzpracovani();
                } else if (predzpracovani) {
                    predzprac.storno = true;
                    initOkno.setVisible(true);
                    spustitInicializaci();
                } else {
                    alg.setStorno(true);
                    task.cancel(true);
                }

                timer.stop();


            } else if (task.isCancelled()) {
                progressMonitor.close();
                if (inicializace) {
                    dokonciPredzpracovani();
                } else if (predzpracovani) {
                    spustitInicializaci();
                } else {
                    algoritmusOkno.nastav();
                }
                timer.stop();

            } else if (task.isDone()) {
                timer.stop();
                progressMonitor.close();
                if (inicializace) {
                    spustitVysledky();
                } else if (predzpracovani) {
                    dokonciPredzpracovani();
                }
            } else {
                if (predzpracovani) {
                    progressMonitor.setNote(predzprac.getPostupText());

                    progressMonitor.setProgress(predzprac.getPostup());
                } else {
                    progressMonitor.setNote(alg.getPoznamka());

                    progressMonitor.setProgress(alg.getProgres());
                }
            }
        }
    }

/**
 * Starts the pre-processing. Sets the progress monitor and executes the task. 
 */
    public void zacatekPredzpracovani() {
        krok = false;
        predzpracovani = true;
        beh = false;
        inicializace = false;
        task = new Task();
        progressMonitor = new ProgressMonitor(initOkno,
                "Preprocessing is running:",
                "a", 0, 1000);

        progressMonitor.setProgress(0);

        progressMonitor.setMillisToDecideToPopup(0);
        initOkno.setVisible(
                false);
        timer.start();
        task.execute();
    }

/**
 * Executes the algorithm step. Sets the progress monitor and executes the task. 
 */
    public void krok() {
        krok = true;
        predzpracovani = false;
        beh = false;
        inicializace = false;
        alg.setStorno(false);
        task = new Task();
        progressMonitor = new ProgressMonitor(algoritmusOkno,
                "Algorithm step is running....",
                "a", 0, 100);

        progressMonitor.setProgress(0);
        progressMonitor.setMillisToDecideToPopup(0);
        algoritmusOkno.setEnabled(false);
        timer.start();
        task.execute();

    }

    /**
     * Executes the algorithm. Sets the progress monitor and executes the task. 
     */
    public void beh() {
        krok = false;
        predzpracovani = false;
        beh = true;
        inicializace = false;
        alg.setStorno(false);
        task = new Task();

        progressMonitor = new ProgressMonitor(algoritmusOkno,
                "Algorithm is running",
                "a", 0, 100);

        progressMonitor.setProgress(0);

        progressMonitor.setMillisToDecideToPopup(0);
        algoritmusOkno.setEnabled(false);
        timer.start();
        task.execute();
    }

    /**
     * Executed algorithm initialization. Sets the progress monitor and 
     * executes the task. 
     */
    public void inicializace() {

        krok = false;
        predzpracovani = false;
        beh = false;
        inicializace = true;
        try {
            this.alg = new Algoritmus(data.getPocetTrid());
            // algoritmusOkno.setVybranyShluk(null);
        } catch (Exception e) {
            Main.vyridVyjimku(e);
        }
        alg.setStorno(false);
        task = new Task();
        progressMonitor = new ProgressMonitor(algoritmusOkno,
                "Algorithm initialization",
                "a", 0, 100);

        progressMonitor.setProgress(0);
        progressMonitor.setMillisToDecideToPopup(0);
        timer.start();
        task.execute();
        predzpracovaniOkno.setVisible(false);

    }

       /**
        * Generates a CSV file from the information about clusters.
        */
    public void csv(File soubor) {
        try {
            IO.stringNaSoubor(alg.vytvorCSV(), soubor);
        } catch (Exception e) {
            Main.vyridVyjimku(e);
        }
    }
}
