/* Text clustering method, SKWIC algorithm implementation
 * Controller for command line parameters processing
 * Authors: Roman Milos, Vladimir Bartik, May 2011
 */
package dip.kontroler;

import dip.Main;
import dip.model.Data;
import dip.model.IO;
import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Iterator;
import javax.xml.bind.JAXBException;

/**
 * Controller for command line parameters processing
 */
public class Parametry {

    private String jmeno;
    private Data data;
    private boolean gui = false;
    //private String dataDir;


    public boolean isGui() {
        return gui;
    }

    public Parametry() {
        data = Data.getInstance();
    }

    /**
     * Main parsing method. Parses all the parameters and modifies the data
     * according to them.
     */
    public void nacti(String[] args) throws Exception {
        String parametr;
        if (args.length == 0|| args[0].charAt(0)==13) {
            gui = true;
            return;
        }

        Iterator p = Arrays.asList(args).iterator();
        while (p.hasNext()) {

            parametr = p.next().toString();
            if (parametr.charAt(parametr.length()-1)==13){
               parametr= parametr.substring(0,parametr.length()-1 );
              
            }
            parametr.toLowerCase();

            if (parametr.equals("-!stemm")) {
                data.setAplikovatStemming(false);
            } else if (parametr.equals("-stemm")) {
                data.setAplikovatStemming(false);
            } else if (parametr.equals("-!termy1dok")) {
                data.setMazatTermyJednohoDokumentu(false);
            } else if (parametr.equals("-termy1dok")) {
                data.setMazatTermyJednohoDokumentu(false);
            } else if (parametr.equals("-tf")) {
                data.setVahovani(Data.VahovaciMetody.TF);
            } else if (parametr.equals("-idf")) {
                data.setVahovani(Data.VahovaciMetody.IDF);
            } else if (parametr.equals("-tfidf")) {
                data.setVahovani(Data.VahovaciMetody.TF_IDF);
            } else if (parametr.equals("-!cisla")) {
                data.setMazatCisla(false);

            } else if (parametr.equals("-cisla")) {
                data.setMazatCisla(false);
            } else if (parametr.equals("-data")) {
                if (!p.hasNext()) {
                    throw new Exception("Incorrect parameters, bad folder name");
                } else {
                    try {
                        data.setDataDirString(p.next().toString());

                    } catch (Exception e) {
                        throw new Exception("Incorrect parameter: " + e.getMessage());
                    }
                }
            } else if (parametr.equals("-import")) {
                if (!p.hasNext()) {
                    throw new Exception("Incorrect parameters, bad import XML file name");
                } else {
                    try {
                        jmeno = (String) p.next();

                    } catch (Exception e) {
                        throw new Exception("Incorrect parameter: " + e.getMessage());
                    }
                }
            } else if (parametr.equals("-stop")) {
                if (!p.hasNext()) {
                    throw new Exception("Incorrect parameters, bad stop words file name");
                } else {
                    try {
                        data.setStopListString(p.next().toString());
                        data.setAplikovatStopList(true);
                    } catch (Exception e) {
                        throw new Exception("Incorrect parameters: " + e.getMessage());
                    }
                }
            } else if (parametr.equals("-maxTermu")) {
                if (!p.hasNext()) {
                    throw new Exception("Incorrect parameters, bad number of terms");
                } else {
                    try {
                        int trid = new Integer(p.next().toString());
                        if (trid <= 10) {
                            throw new Exception();
                        }
                        data.setPocetTrid(trid);
                    } catch (Exception e) {
                        throw new Exception("Incorrect parameter: Maximum count of terms must be more than 10.");
                    }
                }
            } else if (parametr.equals("-trid")) {
                if (!p.hasNext()) {
                    throw new Exception("Incorrect parameter: bad number of classes");
                } else {
                    try {
                        int trid = new Integer(p.next().toString());
                        if (trid == 0) {
                            throw new Exception();
                        }
                        data.setPocetTrid(trid);
                    } catch (Exception e) {
                        throw new Exception("Incorrect number of clases");
                    }
                }
            } else if (parametr.equals("-konstanta")) {
                if (!p.hasNext()) {
                    throw new Exception("Incorrect parameter: bad constant");
                } else {
                    try {
                       double konstanta = new Double(p.next().toString());
                        if (konstanta == 0) {
                            throw new Exception();
                        }
                        data.setKonstanta(konstanta);
                    } catch (Exception e) {
                        throw new Exception("Incorrect constant");
                    }
                }
            } else {
                throw new Exception("Unknown parameter: " + parametr + " ");

            }
        }
        gui = true;
        if (jmeno != null && jmeno.length() > 0) {
            importXml();
        }
    }

/**
 * Import of XML file given as a command line parameter.
 */
    private void importXml() {

        try {
            File soubor = new File(jmeno);
            IO.importuj(soubor);
        } catch (IOException ex) {
            Main.vyridVyjimku(new Exception("Unable to get import file"));
            return;
        } catch (JAXBException ex) {
            Main.vyridVyjimku(new Exception("Incorrect XML file format"));
            return;
        } catch (Exception ex) {
            ex.printStackTrace();
            Main.vyridVyjimku(new Exception(ex));
            return;
        }
    }
}

