/* Text clustering method, SKWIC algorithm implementation
 * Model class used to implement input/output operations.
 * Authors: Roman Milos, Vladimir Bartik, May 2011
 */
package dip.model;

import dip.Main;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;

/**
 * Model class used to implement input/output operations.
 * @author Roman
 */
public class IO {

    //Size of buffer
    private static final int bufferVelikost = 1024;

    /**
     * Returns information about (reduced) format of double numbers.
     * @return double format with 3 digits
     */
    public static final DecimalFormat formatDouble() {
        return new DecimalFormat("0.000");
    }

    /**
     * Export of data. Generates XML file from actual data stored. 
     * @param soubor a target file to generate data
     * @throws IOException
     */
    public static void export(File soubor) throws IOException {
        if (soubor == null) {
            throw new IOException("Unable to write into file");
        } else if (!soubor.exists()) {
            soubor.createNewFile();
        }
        //get data
        Data data = Data.getInstance();
        //prepare data
        data.pripravNaExport();

        //Create XML a write
        JAXBContext jaxbContext;
        try {
            FileWriter fw = new FileWriter(soubor);

            jaxbContext = JAXBContext.newInstance(Data.class);
            Marshaller m = jaxbContext.createMarshaller();
            m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            m.marshal(Data.getInstance(), fw);
            //System.err.println("hotovo");
        } catch (JAXBException e) {
            //e.printStackTrace();
            Main.vyridVyjimku(new Exception("Unable to store into file"));
        }
    }

    /**
     * Import data. Actual data contents are overwritten with data from XML file.
     * @param soubor an input XML file
     * @throws IOException
     * @throws JAXBException
     * @throws Exception
     */
    public static void importuj(File soubor) throws IOException, JAXBException, Exception {
        Data data = Data.getInstance();
        if (soubor == null || !soubor.isFile()) {
            throw new IOException("error");
        }

        JAXBContext jc = JAXBContext.newInstance(Data.class);

        Unmarshaller u = jc.createUnmarshaller();
        //Get data from XML
        Data parse = (Data) u.unmarshal(soubor);

        //copy data from XML and check errors
        data.setAplikovatStemming(parse.isAplikovatStemming());
        data.setMazatCisla(parse.isMazatCisla());
        data.setMazatTermyJednohoDokumentu(parse.isMazatTermyJednohoDokumentu());
        data.setDataFile(parse.getDataFile());
        if (parse.getMaximalneTermu() > 10) {
            data.setMaximaleTermu(parse.getMaximalneTermu());
        }
        if (parse.getStopListFile() != null) {
            data.setAplikovatStopList(true);
        }
        data.setStopListFile(parse.getStopListFile());
        data.setVahovani(parse.getVahovani());
        data.setSlovaDokumentyKolekce(parse.getSlovaDokumentyKolekce());
        data.setDokumenty(parse.getDokumenty());
        data.setPocetTrid(parse.getPocetTrid());
        data.setKonstanta(parse.getKonstanta());
        if (data.getKonstanta() == 0) {
            throw new Exception("Incorrect constant");
        }
        data.setKategorieList(parse.getKategorieList());
        data.opravPoImportu();

    }

    /**
     * Converts given file into a string. 
     * @param soubor a given input file
     * @return string containing the contents of file
     * @throws java.io.IOException
     */
    public static String souborNaString(File soubor)
            throws java.io.IOException {

        StringBuffer fileData = new StringBuffer(bufferVelikost);
        BufferedReader reader = new BufferedReader(new FileReader(soubor));
        char[] buffer = new char[bufferVelikost];
        int znaku = 0;
        while ((znaku = reader.read(buffer)) != -1) {
            fileData.append(buffer, 0, znaku);
        }
        reader.close();
        return fileData.toString();
    }

    /**
     * Stores a given string into a file.
     * @param data an input string
     * @param soubor an output file
     * @throws Exception
     */
    public static void stringNaSoubor(String data, File soubor) throws Exception {
        if (soubor == null) {
            throw new Exception("Unable to write into file");
        } else if (!soubor.exists()) {
            soubor.createNewFile();
        }
        try {
            FileWriter writer = new FileWriter(soubor);
            writer.write(data);
            writer.close();
        } catch (Exception e) {
            throw new Exception("Unable to write into file");
        }

    }

    /**
     * Creates a text file containing all documents stored in the application data. 
     * @return an output string, which will form a file
     * @throws Exception
     */
    public static String exportDokumentuTXT() throws Exception {
        try {
            StringBuffer retezec = new StringBuffer();
            Data data = Data.getInstance();
            for (Dokument d : data.getDokumenty()) {
                retezec.append(d.getKategorie() + "\t");
                for (Slovo s : d.getSlova().values()) {
                    for (int i = 0; i < s.getPocet(); i++) {
                        retezec.append(s.getJmeno() + " ");
                    }
                }
                retezec.append("\n");

            }
            return retezec.toString();
        } catch (Exception e) {
            throw new Exception("Unable to to create data for export");

        }
    }

    /**
     * Stores all documents into a given file.
     * @param soubor an output file
     * @throws Exception
     */
    public static void ulozDokumenty(File soubor) throws Exception {
        stringNaSoubor(exportDokumentuTXT(), soubor);
    }
}
