/* Text clustering method, SKWIC algorithm implementation
 * Model class with the implementation of the algorithm
 * Authors: Roman Milos, Vladimir Bartik, May 2011
 */
package dip.model.algoritmus;

import dip.model.Data;
import dip.model.Dokument;
import dip.model.Slovo;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Random;

/**
 * Model class with the implementation of the algorithm
 * @author Roman
 */
public class Algoritmus {

    //necessary variables
    private double konstanta = 1;//.0005;
    private final int VELKE_CISLO = 2147000000;
    private long dobaBehu = 0;
    private List<Shluk> shluky;
    private List<DokumentShluku> kPresunuti;
    private Data data;
    private Random nahodne;
    private int pocetShluku;
    private static double vahaSlova;
    private HashSet<String> stred;
    private boolean hotovo = false;
    private boolean storno = false;
    private int iterace = 0;
    private Validace validace;
    private int progres;
    private String poznamka = "";
    private double prvni;
    private double druha;
    private int zaporne = 0;

    /**
     * Gets the actual value of negative weight of cluster. 
     * @return an integer value with negative weight
     */  
    public int getZaporne() {
        return zaporne;
    }
    
    /**
     * Gets the actual value of time spent by the run of algorithm. 
     * @return a value time complexity
     */
    public long getDobaBehu() {
        return dobaBehu;
    }
    
    /**
     * Sets the actual value of time spent by the run of algorithm. 
     * @param dobaBehu a value time complexity
     */
    public void setDobaBehu(long dobaBehu) {
        this.dobaBehu = dobaBehu;
    }

    /**
     * Gets the value distance between a document and cluster center. 
     * @return a double value with the value
     */    
    public double getPrvni() {
        return prvni;
    }

    /**
     * Gets the actual "delta" value. 
     * @return a double value with the value
     */  
    public double getDruha() {
        return druha;
    }
    
    /**
     * Gets the value of tag, which decides, if the algorithm should be stopped. 
     * @return a boolean value of the tag
     */
    public boolean isStorno() {
        return storno;
    }

    /**
     * Sets the value of tag, which decides, if the algorithm should be stopped. 
     * @param storno a boolean value of the tag
     */
    public void setStorno(boolean storno) {
        this.storno = storno;
    }
    
    /**
     * Gets the actual message about actual activity of the algorithm. 
     * @return a string with the message
     */
    public String getPoznamka() {
        return poznamka;
    }
    
    /**
     * Sets the actual message about actual activity of the algorithm. 
     * @param poznamka a string with the message
     */
    public void setPoznamka(String poznamka) {
        this.poznamka = poznamka;
    }
    
    /**
     * Gets the actual progress of the algorithm. 
     * @return a number representing the progress
     */
    public int getProgres() {
        return progres;
    }

    /**
     * Sets the actual progress of the algorithm. 
     * @param progres a number representing the progress
     */
    public void setProgres(int progres) {
        this.progres = progres;
    }

    /**
     * Gets the value of tag referencing about algorithm finishing. 
     * @return a boolean value of the tag
     */    
    public boolean isHotovo() {
        return hotovo;
    }

    /**
     * Sets the value of tag referencing about algorithm finishing. 
     * @param hotovo a boolean value of the tag
     */
    public void setHotovo(boolean hotovo) {
        this.hotovo = hotovo;
    }
    
    /**
     * Gets the number of actual iteration of the algorithm. 
     * @return number of actual iteration
     */
    public int getIterace() {
        return iterace;
    }
    
    /**
     * Sets the number of actual iteration of the algorithm. 
     * @param iterace number of actual iteration
     */
    public void setIterace(int iterace) {
        this.iterace = iterace;
    }
    
    /**
     * Gets the actually used instance of class responsible for results 
     * validation. 
     * @return instance of Validace class
     */  
    public Validace getValidace() {
        return validace;
    }

    /**
     * Sets the instance of class responsible for results validation. 
     * @param validace instance of Validace class
     */    
    public void setValidace(Validace validace) {
        this.validace = validace;
    }

    /**
     * Gets the initial weight of each term. 
     * @return number with the initial weight
     */    
    public static double getVahaTermu() {
        return vahaSlova;
    }
    
    /**
     * Gets the input data for the algorithm. 
     * @return data for the algorithm
     */ 
    public Data getData() {
        return data;
    }

    /**
     * Sets the input data for the algorithm. 
     * @param data data for the algorithm
     */ 
    public void setData(Data data) {
        this.data = data;
    }
    
    /**
     * Gets the actual random document forming a cluster
     * @return a Random class instance
     */
    public Random getNahodne() {
        return nahodne;
    }
    
    /**
     * Gets the count of clusters created by the algorithm. 
     * @return count of clusters
     */  
    public int getPocetShluku() {
        return pocetShluku;
    }

    /**
     * Gets the list of clusters created by the algorithm. 
     * @return list of clusters
     */    
    public List<Shluk> getShluky() {
        return shluky;
    }

    /**
     * Computes the error function. 
     */
    public void spoctiChybu() {
        Shluk shluk;
        prvni =
                0;
        druha =
                0;
        double tmp = 0;

        for (int i = 0; i <
                pocetShluku; i++) {
            shluk = shluky.get(i);
            for (DokumentShluku ds : shluk.getDokumentyShluku()) {
                prvni += ds.getVzdalenostDokumentu();
            }

            tmp = 0;
            if (shluk.getDokumentyShluku().size() > 0) {
                for (int j = 0; j <
                        shluk.getVahy().length; j++) {
                    tmp += shluk.getVahy()[i] * shluk.getVahy()[i];
                }

                druha += shluk.getDelta() * tmp;
            }

        }


    }

    /**
     * Constructor of the algorithm. 
     * @param pocetShluku number of clusters
     * @throws Exception
     */
    public Algoritmus(int pocetShluku) throws Exception {

        this.pocetShluku = pocetShluku;
        data =
                Data.getInstance();
        shluky =
                new ArrayList<Shluk>(pocetShluku);
        kPresunuti =
                new ArrayList<DokumentShluku>(pocetShluku);
        vahaSlova =
                1.0 / data.getSlovaDokumenty().size();
        stred =
                new HashSet<String>(pocetShluku);
        //  nahodne = new Random(data.getDokumenty().size());
        nahodne =
                new Random();
        if (this.pocetShluku > data.getDokumenty().size()) {
            throw new Exception("Lack of documents for cluster centres");
        }

        konstanta = data.getKonstanta();
        // inicializace();
        //   beh();


    }

    /**
     * Initialization of the algorithm.
     * @throws Exception
     */
    public void inicializace() throws Exception {
        progres = 0;
        long cas = System.currentTimeMillis();
        poznamka =
                "Creating clusters";
        inicializujStredyShluku();

        progres =
                10;
        poznamka =
                "Assigning documents into clusters";
        inicializujDokumentyShluku();

        progres =
                80;
        presunDokumenty();

        progres =
                85;
        poznamka =
                "Center actualization";
        aktualizujStredy();

        progres =
                88;
        aktualizujVzdalenostiDokumentu();

        progres =
                92;
        aktualizujDeltu();

        progres =
                94;

        validace =
                new Validace(this);
        validace.naplnTabulku();

        progres =
                99;
        spoctiChybu();

        dobaBehu +=
                System.currentTimeMillis() - cas;

    }

    /**
     * One step of the algorithm. 
     * @return boolean value of successfulness
     */
    public boolean krok() {
        long cas = System.currentTimeMillis();
        progres =
                0;
        if (!hotovo) {

            iterace++;
            poznamka =
                    "Iteration " + iterace + ": Distance counting";
            aktualizujVzdalenostiDokumentu();

            progres =
                    5;
            poznamka =
                    "Iteration " + iterace + ": Weights counting";
            aktualizujVahy();

            progres =
                    15;
            poznamka =
                    "Iteration " + iterace + ": Correcting distances";
            aktualizujVzdalenostiDokumentu();

            progres =
                    20;

            if (storno) {
                iterace--;
                return false;

            }


            poznamka = "Iteration " + iterace + ": Correcting distribution";
            prepocitejRozdeleniDokumentu();

            if (kPresunuti.isEmpty()) {
                hotovo = true;
                return false;

            }


            if (storno) {
                return false;
            }

            poznamka = "Iteration " + iterace + ": Moving documents";

            presunDokumenty();

            progres =
                    70;
            poznamka =
                    "Iteration " + iterace + ": Centers actualization";
            aktualizujStredy();

            progres =
                    80;
            poznamka =
                    "Iteration " + iterace + ": Error actualization";
            aktualizujVzdalenostiDokumentu();

            aktualizujDeltu();

            spoctiChybu();

            progres =
                    90;
            poznamka =
                    "Iteration " + iterace + ": Finishing";
            dobaBehu +=
                    System.currentTimeMillis() - cas;
        }

        validace.naplnTabulku();
        progres = 99;
        return true;
    }

    /**
     * The whole run of the algorithm, steps are subsequently executed.
     */
    public void beh() {

        while (!hotovo) {
            krok();

            if (storno) {
                break;
            }

        }


        validace.naplnTabulku();
    }

    /**
     * Actualizes the weights of clusters, adds negative weights.
     */
    private void aktualizujVahy() {
        Shluk shluk;
        for (int i = 0; i <
                pocetShluku; i++) {
            shluk = shluky.get(i);
            zaporne += shluk.spoctiVahy();

        }

    }

    /**
     * Creates clusters with random centers, which should be unique.
     * @throws Exception
     */
    private void inicializujStredyShluku() throws Exception {
        Shluk shluk;

        while (stred.size() < pocetShluku) {
            Dokument dokument = ziskejNahodnyDokument();
            if (!stred.contains(dokument.getJmeno())) {
                shluk = new Shluk(dokument);
                shluky.add(shluk);
                shluk.setStred(dokument.getJmeno());

                stred.add(dokument.getJmeno());
            }

        }
        //   presunDokumenty();
    }

    /**
     * Makes a new distribution of documents into clusters.
     */
    private void prepocitejRozdeleniDokumentu() {

        Shluk shluk;

        for (int i = 0; i <
                pocetShluku; i++) {
            progres = 20 + (int) (50 * (double) i / pocetShluku);
            shluk =
                    shluky.get(i);
            for (DokumentShluku dokumentShluku : shluk.getDokumentyShluku()) {
                najdiMinialniVzdalenostOdShluku(dokumentShluku);
            }

        }
        ziskejDokumentyKPresunuti();

    }

    /**
     * The first initiative distribution of documents into clusters. 
     */
    private void inicializujDokumentyShluku() {
        DokumentShluku dokumentShluku;
        int pocet = data.getDokumenty().size() / 70;
        int i = 0;
        for (Dokument d : data.getDokumenty()) {
            progres = 10 + i / pocet;
            i++;

            dokumentShluku =
                    new DokumentShluku(d, pocetShluku);
            najdiMinialniVzdalenostOdShluku(dokumentShluku);
            kPresunuti.add(dokumentShluku);
        }

    }

    /**
     * Finds a cluster, which has minimum distance from a given document.
     * @param dok a given document
     */
    private void najdiMinialniVzdalenostOdShluku(DokumentShluku dok) {

        int idGlobalniVaha;
        int min = 0;
        double minimum = VELKE_CISLO;
        double suma = 0, Vik, Cik;
        Shluk shluk;

        for (int i = 0; i <
                pocetShluku; i++) {

            shluk = shluky.get(i);

            suma =
                    shluk.getVzdalenostPrazdnehoDokumentu();

            for (Slovo s : dok.getDokument().getSlova().values()) {
                idGlobalniVaha = data.getSlovaDokumenty().get(s.getJmeno()).getId();
                Vik =
                        shluk.getVahy()[idGlobalniVaha];
                Cik =
                        shluk.getCentrum()[idGlobalniVaha];

                suma +=
                        ((vahaSlova - (s.getVaha() * Cik)) * Vik) - Vik * vahaSlova;
            }

            if (minimum > suma) {
                min = i;
                minimum =
                        suma;
            }

        }
        dok.setShluk(shluky.get(min));
        //  dok.setIdShluku(min);
    }


    /**
     * Finds documents, which should be moved to another cluster, according to
     * its distance to clusters. These documents form a list of moved documents.
     */
    private void ziskejDokumentyKPresunuti() {
        Shluk shluk;

        DokumentShluku d;

        List<DokumentShluku> tmp = new ArrayList<DokumentShluku>();
        for (int i = 0; i <
                pocetShluku; i++) {
            shluk = shluky.get(i);

            for (DokumentShluku ds : shluk.getDokumentyShluku()) {
                if (!ds.getShluk().equals(shluk)) {
                    tmp.add(ds);
                    kPresunuti.add(ds);
                    d = ds;

                }
            }

            shluk.getDokumentyShluku().removeAll(tmp);
            tmp.clear();
        }

    }
    
    /**
     * Moves documents (from a list of documents to be moved) between clusters.
     */
    private void presunDokumenty() {
        Shluk shluk;
        for (DokumentShluku d : kPresunuti) {
            shluk = d.getShluk();
            shluk.getDokumentyShluku().add(d);
        }

        kPresunuti.clear();
    }

    /**
     * Makes actualization of distances in all clusters.
     */
    private void aktualizujVzdalenostiDokumentu() {
        Shluk shluk;
        for (int i = 0; i <
                pocetShluku; i++) {
            shluk = shluky.get(i);
            // shluk.aktualizujVzdalenostPrazdnehoDokumentu();
            shluk.aktualizujVzdalenostiDokumentu();
        }

    }

    /**
     * Computes the delta coefficient for each cluster.  
     */
    private void aktualizujDeltu() {
        double sumaV2, suma;
        Shluk shluk;

        for (int i = 0; i <
                shluky.size(); i++) {
            sumaV2 = 0;

            shluk =
                    shluky.get(i);
            for (int j = 0; j <
                    shluky.get(i).getVahy().length; j++) {
                sumaV2 += shluk.getVahy()[j] * shluk.getVahy()[j];
            }

            suma = 0;
            for (DokumentShluku d : shluk.getDokumentyShluku()) {
                suma += d.getVzdalenostDokumentu();
            }

            suma /= sumaV2;
            shluk.setDelta(konstanta * suma);

        }

    }

    /**
     * Generates random number and returns a random document according to it. 
     * @return random document
     * @throws Exception
     */
    private Dokument ziskejNahodnyDokument() throws Exception {
        try {
            int index = nahodne.nextInt(data.getDokumenty().size() - 1);
            return data.getDokumenty().get(index);
        } catch (Exception e) {
            throw new Exception("Unable to get enough random documents");
        }

    }

    /**
     * Makes actualization of cluster centers. 
     */
    private void aktualizujStredy() {
        for (int i = 0; i <
                pocetShluku; i++) {
            shluky.get(i).obnovStredy();
        }

    }

    /**
     * Creates a CSV file with information about all clusters created by 
     * the algorithm. 
     * @return string forming a CSV file
     */
    public String vytvorCSV() {
        StringBuffer info = new StringBuffer();
        StringBuffer csv = new StringBuffer();
        Shluk shluk;

        List<Slovo> klicoveSlova;
        for (int i = 0; i <
                pocetShluku; i++) {
            shluk = shluky.get(i);
            info.append("," + i);
            klicoveSlova =
                    shluk.getKlicovaSlova();
            for (int j = 0; j <
                    10 && j < klicoveSlova.size(); j++) {
                info.append("," + klicoveSlova.get(j).getJmeno());
            }

            for (DokumentShluku ds : shluk.getDokumentyShluku()) {
                csv.append(ds.getDokument().getJmeno() + info + "\n");
            }

            info.setLength(0);
        }

        return csv.toString();
    }

    /**
     * Returns an identifier of cluster according to its reference. 
     * @param shluk cluster reference
     * @return
     */
    public int getIdShluku(Shluk shluk) {
        for (int i = 0; i <
                pocetShluku; i++) {
            if (shluky.get(i).equals(shluk)) {
                return i;
            }
        }
        return 0;
    }
}
