/* Text clustering method, SKWIC algorithm implementation
 * Model class of a document inside a cluster
 * Authors: Roman Milos, Vladimir Bartik, May 2011
 */
package dip.model.algoritmus;

import dip.model.Data;
import dip.model.Dokument;
import dip.model.Slovo;

/**
 * Model class of a document inside a cluster
 * @author Roman
 */
public class DokumentShluku {

    //reference to a document
    private Dokument dokument;
    //distances between terms of a document
    private double[] Dkw;
    //overall distance between a document and cluster centre
    private double vzdalenostDokumentu;
    //reference to a cluster
    private Shluk shluk;
    //reference to a dataset
    private Data data = Data.getInstance();
    public double necoSpocitane = 0;

    /**
     * Gets the reference to a cluster. 
     * @return instance of the class referencing a cluster
     */      
    public Shluk getShluk() {
        return shluk;
    }

    /**
     * Sets the reference to a cluster. 
     * @param shluk instance of the class referencing a cluster
     */ 
    public void setShluk(Shluk shluk) {
        this.shluk = shluk;
    }

    /**
     * Gets the overall distance between a document and cluster center. 
     * @return value of the distance
     */ 
    public double getVzdalenostDokumentu() {
        return vzdalenostDokumentu;
    }
    
    /**
     * Sets the overall distance between a document and cluster center. 
     * @param vzdalenostDokumentu value of the distance
     */
    public void setVzdalenostDokumentu(double vzdalenostDokumentu) {
        this.vzdalenostDokumentu = vzdalenostDokumentu;
    }
    
    /**
     * Gets the array of distances between terms of a document. 
     * @return array of distances
     */ 
    public double[] getDkw() {
        return Dkw;
    }

    /**
     * Sets the array of distances between terms of a document. 
     * @param Dkw array of distances
     */ 
    public void setDkw(double[] Dkw) {
        this.Dkw = Dkw;
    }

    /**
     * Sets the value of distance at a given index
     * @param Dkw distance value
     * @param i index
     */
    public void setDkwi(double Dkw, int i) {
        this.Dkw[i] = Dkw;
    }

    /**
     * Returns the weight of a term in a document, if exists, otherwise 
     * returns zero. 
     * @param klic the term
     * @return the weight
     */
    public double getVahuTermu(String klic) {
        if (dokument.getSlova().containsKey(klic)) {
            return dokument.getSlova().get(klic).getVaha();
        }
        return 0.0;

    }

    /**
     * Gets the reference to a document. 
     * @return instance of the class referencing a document
     */ 
    public Dokument getDokument() {
        return dokument;
    }

    /**
     * Sets the reference to a document. 
     * @param dokument instance of the class referencing a document
     */ 
    public void setDokument(Dokument dokument) {
        this.dokument = dokument;
    }

    /**
     * Constructor assigns a document and creates an array of distances.
     * @param dokument a document
     * @param shluku count of clusters
     */
    public DokumentShluku(Dokument dokument, int shluku) {
        this.dokument = dokument;
        Dkw = new double[dokument.getSlova().size()];
    }

    /**
     * Returns the distance value for a given term.
     * @param jmeno term given in a form of string
     * @return distance value
     */
    public double getVzdalenostTermu(String jmeno) {
        Slovo slovo = dokument.getSlova().get(jmeno);
        if (slovo != null) {
            return Dkw[slovo.getId()];
        }
        return Algoritmus.getVahaTermu();
    }

    /**
     * Returns the distance value for a given term.
     * @param id term given in a form of its identifier
     * @return distance value
     */
    public double getVzdalenostTermu(int id) {

        return Dkw[id];
    }

    /**
     * Actualizes the array of distances according to a new cluster center
     */
    public void spoctiVzdalenostiDokumentu() {
        double vahaTermu = 1.0 / shluk.getVahy().length;
        int id, idGlobal;
        //distance of empty document
        vzdalenostDokumentu = shluk.getVzdalenostPrazdnehoDokumentu();
    //for all terms
        for (Slovo s : dokument.getSlova().values()) {
            id = s.getId();
            //first id
            idGlobal = data.getSlovaDokumenty().get(s.getJmeno()).getId();
            Dkw[id] = vahaTermu - s.getVaha() * shluk.getCentrum()[idGlobal];
            //remove an old weight
            vzdalenostDokumentu -= shluk.getVahy()[idGlobal] * vahaTermu;
            //add a weight
            vzdalenostDokumentu += shluk.getVahy()[idGlobal] * Dkw[id];

        }
        //   aktualizujVzdalenost();
    }
}
